package com.example.julian.endlessroll.main.game;

import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.DestroyEffect;
import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.Goal;
import com.example.julian.endlessroll.entities.Obstacle;
import com.example.julian.endlessroll.entities.Star;
import com.example.julian.endlessroll.entities.Terrain;
import com.example.julian.endlessroll.entities.TerrainTile;
import com.example.julian.endlessroll.entities.collision.CollisionDetector;
import com.example.julian.endlessroll.entities.particles.ParticleSystem;
import com.example.julian.endlessroll.entities.textures.TexturePack;
import com.example.julian.endlessroll.entities.tools.Bomb;
import com.example.julian.endlessroll.entities.tools.Tool;
import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.levels.Level;
import com.example.julian.endlessroll.levels.ObstacleData;
import com.example.julian.endlessroll.levels.StarData;
import com.example.julian.endlessroll.main.GameLog;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Created by Julian on 27.11.2015.
 */
public class GameScene extends Scene {

    private CollisionDetector collisionDetector;
    private Goal goal;

    private float goalX;
    private List<Obstacle> obstacles = new ArrayList<>();
    private List<Star> stars = new ArrayList<>();
    private List<Tool> tools = new ArrayList<>();

    public GameScene(TexturePack texturePack, ParticleSystem particleSystem) throws Exception {
        super(texturePack, particleSystem);
        collisionDetector = new CollisionDetector();

        goal = new Goal(textures.goal);
    }

    public void loadLevel(Level level) throws Exception {
        reset();
        lock.lock();
        terrain.loadData(level.getTerrain());
        super.add(goal);
        player.init(terrain.getHeight());
        super.add(player);
        lock.unlock();
        for (ObstacleData data : level.getObstacles())
            addObstacle(data);
        for (StarData data : level.getStars()) {
            if (data.getIndex() == 1 && level.isFirstStarCollected())
                continue;
            if (data.getIndex() == 2 && level.isSecondStarCollected())
                continue;
            if (data.getIndex() == 3 && level.isThirdStarCollected())
                continue;
            addStar(data);
        }
        goalX = level.getGoalX();
        goal.setGoalX(goalX);
        GameLog.d("Level " + level.getNumber() + " successfully loaded");
    }

    private void reset() {
        lock.lock();
        super.clear();
        lock.unlock();
        obstacles.clear();
        tools.clear();
        cameraX = 0;
        background.resetPosition();
    }

    public void onStarCollision(Star collisionStar) {
        Iterator<Star> iter = stars.iterator();
        while (iter.hasNext()) {
            Star star = iter.next();
            if (star.equals(collisionStar)) {
                star.destroy(DestroyEffect.STAR_EXPLOSION);
                iter.remove();
            }
        }
    }

    public void addObstacle(ObstacleData data) {
        Obstacle obstacle = new Obstacle(textures.obstacle, data, terrain.getHeight());
        lock.lock();
        super.add(obstacle);
        obstacles.add(obstacle);
        lock.unlock();
    }

    public void addStar(StarData data) {
        Star star = new Star(textures.star, data);
        lock.lock();
        super.add(star);
        stars.add(star);
        lock.unlock();
    }

    public void addTool(ToolType type, float screenX, float screenY) throws Exception {
        Vector position = calcWorldFromScreenCoords(screenX, screenY);
        Tool tool = type.newInstance(position, particleSystem);
        if (tool == null)
            throw new Exception("Current ToolType(" + type + ") returns null at method newInstance()");
        lock.lock();
        super.add(tool);
        tools.add(tool);
        lock.unlock();
    }


    @Override
    public void update(Timer timer) {
        super.update(timer);
        for (Tool tool : tools) {
            tool.update(timer);
            if (tool instanceof Bomb) {
                Bomb bomb = (Bomb) tool;
                if (bomb.isExploding())
                    bomb.explode(obstacles, collisionDetector);
            }
        }
    }

    @Override
    protected void removeEntityFromAllLists(Entity entity) {
        GameLog.i("REMOVE FROM ALL LISTS");
        if (!entity.isDestroyed())
            entity.destroy(null);
        if (entity instanceof Tool)
            tools.remove(entity);
        if (entity instanceof Obstacle)
            obstacles.remove(entity);
    }

    public Terrain getTerrain() {
        return terrain;
    }

    public List<Tool> getTools() {
        return tools;
    }

    public List<Obstacle> getObstacles() {
        return obstacles;
    }

    public List<Star> getStars() {
        return stars;
    }

    public float getGoalX() {
        return goalX;
    }
}
