package com.example.julian.endlessroll.main.game;

import android.support.annotation.Nullable;

import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.Obstacle;
import com.example.julian.endlessroll.entities.Star;
import com.example.julian.endlessroll.entities.Terrain;
import com.example.julian.endlessroll.entities.TerrainTile;
import com.example.julian.endlessroll.entities.collision.CollisionDetector;
import com.example.julian.endlessroll.entities.collision.collisionData.EntityCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.ObstacleCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.PlayerCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.ToolCollisionData;
import com.example.julian.endlessroll.entities.collision.geometry.Circle;
import com.example.julian.endlessroll.entities.tools.Bomb;
import com.example.julian.endlessroll.entities.tools.Tool;
import com.example.julian.endlessroll.main.GameLog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Julian on 27.11.2015.
 */
public class Physics {

    public final float GRAVITY_FORCE = .00004f;
    private CollisionDetector detector;

    public Physics() {
        detector = new CollisionDetector();
    }

    public void applyGravity(GameScene scene) {
        scene.getPlayer().getMovement().y -= GRAVITY_FORCE;
        for (Tool tool : scene.getTools()) {
            if (tool.isFloating())
                continue;
            if (tool instanceof Bomb) {
                for (Obstacle obstacle : scene.getObstacles()) {
                    if (detector.quadQuadCollision(obstacle, tool)) {
                        GameLog.i("collision");
                        tool.setFloating(true);
                        tool.getMovement().y = 0;
                        break;
                    }
                }
                if (tool.isFloating())
                    continue;
            }
            float terrainHeight = getTerrainHeight(tool, scene.getTerrain());
            Obstacle toolIsOver = getObstacleToolIsOver(tool, scene.getObstacles());

            float orientingHeight = terrainHeight;
            if (toolIsOver != null) {
                if (toolIsOver.getBottomEdge() < tool.getTopEdge())
                    orientingHeight = Math.max(toolIsOver.getTopEdge(), terrainHeight);
                else{
                    orientingHeight = terrainHeight;
                }
            }

            if (tool.getBottomEdge() > orientingHeight) {
                tool.getMovement().y -= GRAVITY_FORCE * 2;
            } else {
                tool.getMovement().y = 0;
                tool.setToTerrain(orientingHeight);
            }
        }
    }

    @Nullable
    private Obstacle getObstacleToolIsOver(Entity tool, List<Obstacle> obstacles) {
        Map<Float, Obstacle> isOver = new HashMap<>();
        for (Obstacle obstacle : obstacles) {
            if ((tool.getLeftEdge() >= obstacle.getLeftEdge() && tool.getLeftEdge() <= obstacle.getRightEdge()) || (tool.getRightEdge() <= obstacle.getRightEdge() && tool.getRightEdge() >= obstacle.getLeftEdge()))
                isOver.put(obstacle.getTopEdge(), obstacle);
        }
        float max = -100;
        for (float height : isOver.keySet()) {
           if(isOver.get(height).getBottomEdge() < tool.getTopEdge()) {
                max = Math.max(max, height);
            }
        }
        if (max != -100)
            return isOver.get(max);
        return null;
    }

    private float getTerrainHeight(Entity tool, Terrain terrain) {
        for (TerrainTile instance : terrain) {
            if ((tool.getLeftEdge() >= instance.getLeftEdge() && tool.getLeftEdge() <= instance.getRightEdge())
                    || (tool.getRightEdge() <= instance.getRightEdge() && tool.getRightEdge() >= instance.getLeftEdge())
                    || (instance.getLeftEdge() >= tool.getLeftEdge() && instance.getLeftEdge() <= tool.getRightEdge())
                    || (instance.getRightEdge() <= tool.getRightEdge() && instance.getRightEdge() >= tool.getLeftEdge()))
                return terrain.getHeight();
        }
        return -10;
    }

    public PlayerCollisionData getPlayerCollisionData(GameScene scene) {
        EntityCollisionData terrainData = playerCollidesWithTerrain(scene);
        ObstacleCollisionData obstacleData = playerCollidesWithObstacle(scene);
        ToolCollisionData toolData = playerCollidesWithTool(scene);
        EntityCollisionData starData = playerCollidesWithStar(scene);
        return new PlayerCollisionData(terrainData, obstacleData, toolData, starData);
    }

    private EntityCollisionData playerCollidesWithTerrain(GameScene scene) {
        Terrain terrain = scene.getTerrain();
        for (TerrainTile terrainTile : terrain) {
            EntityCollisionData data = detector.playerEntityCollision(scene.getPlayer(), terrainTile);
            if (data.isCollision())
                return data;
        }
        return new EntityCollisionData(null, null);
    }

    private ObstacleCollisionData playerCollidesWithObstacle(GameScene scene) {
        List<EntityCollisionData> collisions = new ArrayList<>();
        for (Obstacle obstacle : scene.getObstacles()) {
            EntityCollisionData data = detector.playerEntityCollision(scene.getPlayer(), obstacle);
            if (data.isCollision())
                collisions.add(data);
        }
        return new ObstacleCollisionData(collisions);
    }

    private ToolCollisionData playerCollidesWithTool(GameScene scene) {
        List<Tool> tools = new ArrayList<>();
        Circle circle = new Circle(scene.getPlayer());
        for (Tool tool : scene.getTools()) {
            if (detector.circleToolCollision(circle, tool.getCollisionBounds()))
                tools.add(tool);
        }
        return new ToolCollisionData(tools);
    }

    private EntityCollisionData playerCollidesWithStar(GameScene scene) {
        for (Star star : scene.getStars()) {
            EntityCollisionData data = detector.playerEntityCollision(scene.getPlayer(), star);
            if (data.isCollision())
                return data;
        }
        return new EntityCollisionData(null, null);
    }
}
